import pandas as pd
import glob
import os
from datetime import datetime
import pickle
import random
import matplotlib.pyplot as plt
import numpy as np
import re
from datetime import timedelta


# 
def get_all_files(path_to_measurements):
  # 
  list_files = glob.glob(path_to_measurements)
  # 
  df_list = []
  for i in range(len(list_files)):
    df_temp = pd.read_csv(list_files[i],sep='\t',skiprows=(0,1,2),header=(30))
    df_list.append(df_temp)
  # 
  return df_list, list_files

def find_header_value(field, filename):
  # Get first 6 lines
  with open(filename) as file:
      lines = [next(file) for x in range(30)]
  value = None
  for line in lines:
      if line.startswith(field):
          # Get the part of the string after the field name
          end_of_string = line[len(field):]
          string = end_of_string[:-1]
  return string
  
def get_datetime(list_files):
  # 
  date_list = []
  time_list = []
  date_time_list = []
  for i in range(len(list_files)):
    filename = list_files[i]
    field = "Date = "
    date = find_header_value(field, filename)
    date_list.append(date)
    field = "Time = "
    time = find_header_value(field, filename)
    time_list.append(time)
    date_time =  date + ' ' + time
    try:
      datetime_object = datetime.strptime(date_time, '%d/%m/%Y %H:%M:%S')
      date_time_list.append(datetime_object)
    except:
      date_time = date_time.replace("24", "00")
      datetime_object = datetime.strptime(date_time, '%d/%m/%Y %H:%M:%S')
      datetime_object = datetime_object + timedelta(days=1)
      date_time_list.append(datetime_object)
  # 
  return date_list, time_list, date_time_list


def load_ml_model(path_to_model):
  model = pickle.load(open(path_to_model, "rb"))
  return model

def random_sample(df_list,x_columns):
  i = random.randint(0, len(df_list))
  df_temp = df_list[i]
  df_temp = df_temp[x_columns]
  # df_temp = df_temp.iloc[:,[1,2,3,4,5,6,7,8,9,10,11,12]]
  j = random.randint(0, len(df_temp.count()))
  input_data_sample = df_temp.iloc[[j]]
  print(input_data_sample)
  # 
  x = input_data_sample.columns.tolist()
  y = input_data_sample.iloc[0].values.tolist()
  # 
  plt.bar(x, y)
  fig = plt.gcf()
  fig.autofmt_xdate()
  # 
  return input_data_sample

def sample_prediction(input_data_sample, model):
  # 
  print('Prediction: ' + str(model.predict(input_data_sample)))
  print('Prediction Probability: ' + str(model.predict_proba(input_data_sample)))


def calculate_prediction_and_probability(list_files, df_list, x_columns, model):
  # 
  prediction = []
  probability = []
  for i in range(len(list_files)):
    df_temp = df_list[i]
    df_temp = df_temp[x_columns]
    prediction.append([])
    probability.append([])
    for j in range(df_temp.count()[0]):
      input_data_sample = df_temp.iloc[[j]]
      prediction_temp =  int(model.predict(input_data_sample))
      prediction[i].append(prediction_temp)
      probability_temp =  model.predict_proba(input_data_sample)
      probability_temp = float(probability_temp[0][1])
      probability[i].append(probability_temp)
  
  return prediction, probability

def calculate_mean_probability(list_files, probability):
  # 
  mean_probability = []
  std_probability = []
  for i in range(len(list_files)):
    probability_array = np.array(probability[i])
    # 
    mean_probability_temp = np.mean(probability_array)
    mean_probability.append(mean_probability_temp)
    # 
    std_probability_temp = np.std(probability_array)
    std_probability.append(std_probability_temp)
  # 
  return mean_probability, std_probability

def plot_histogram_probabilities(mean_probability):
  # 
  plt.hist(mean_probability, 30) 
  plt.title("histogram") 
  plt.show()

def plot_probability_over_time(date_time_list,mean_probability):
  # 
  plt.style.use('seaborn-colorblind')
  plt.scatter(date_time_list,mean_probability)
  degrees = 70
  plt.axhline(y=0.5, color='r', linestyle='--', linewidth = '1')
  plt.xticks(rotation=degrees)
  plt.xlabel('Time')
  plt.ylabel('Probability of Covid')
  plt.ylim([0,1])
  plt.title('Probability of COVID vs. Time')
  time_now = str(datetime.now())
  plt.savefig('Plot'+time_now +'.png', dpi = 200, bbox_inches='tight')




def get_exp_stage(time, dpps=4, baseline=2, absorb=7, pause=1, desorb=5, flush=23):
    
    # dpps: data point per second

    baseline_time = baseline*dpps
    if time <= baseline_time:
        return 'baseline'
    absorb_time = baseline_time + absorb*dpps
    if time <= absorb_time:
        return 'absorb'
    pause_time = absorb_time + pause*dpps
    if time <= pause_time:
        return 'pause'
    desorb_time = pause_time + desorb*dpps
    if time <= desorb_time:
        return 'desorb'
    flush_time = desorb_time + flush*dpps
    if time <= flush_time:
        return 'flush'
    wait_time = flush_time + flush*dpps
    if time <= wait_time:
        return 'wait'


def rename_columns(df, has_label=False):
    for col in df.columns:
        if 'Sen' in col:
        # print(col[4:])
            new_col = re.findall('\d+', col)[0]
            # print(new_col)
            df.rename(columns={col:f'sensor_{new_col}'}, inplace=True)
    df.rename(columns={'Data Points': 'timesteps'}, inplace=True)
    df.rename(columns={'Humidity (%r.h.)':'humidity'}, inplace=True)
    if has_label:
        df['exp_type'] = df['exp_type'].apply(lambda x: 'Covid' if x == 'COVID' else x)
    return df







def get_label(f):
    exp_name = find_header_value('Name of the experiment =', f)
    if exp_name[-2].isupper():
      return 'Covid'
    else:
      return 'Control'

def get_exp_stage_duration(f):
    baseline = float(find_header_value('Baseline = ', f))
    absorb = float(find_header_value('Absorb = ', f))
    pause = float(find_header_value('Pause = ', f))
    desorb = float(find_header_value('Desorb = ', f))
    flush = float(find_header_value('Flush = ', f))

    return baseline, absorb, pause, desorb, flush

def preprocess_single_file(f, parse_time=True, parse_filename=True, rename_column=True):

    df_temp = pd.read_csv(f,sep='\t',header=(37))
    baseline, absorb, pause, desorb, flush = get_exp_stage_duration(f)
    df_temp['measurement_stage'] = df_temp['Data Points'].apply(get_exp_stage)
    
    if parse_time:
      date = find_header_value('Date = ', f)
      df_temp['date_exp'] = find_header_value('Date = ', f)
      # df_temp['time_start'] = find_header_value('Time = ', f)
      df_temp['time_elapsed'] = df_temp.index / 4
      time_start = find_header_value('Time = ', f)
      time_elapsed = df_temp.index / 4
      timestamp = pd.to_datetime(date + " " + time_start)
      # df_temp['timestamp'] = pd.to_datetime(date + " " + df_temp['time_start'])
      df_temp['datetime_exp'] = pd.to_datetime(date + " " + time_start) + pd.to_timedelta(time_elapsed, unit='s')

    if parse_filename:
      df_temp['filename'] = f.split('/')[-1]
    

    df_temp['result'] = get_label(f)
    
    df_temp['exp_name'] = find_header_value('Name of the experiment = ', f)[1:-1]

    

    if rename_column:
      df = rename_columns(df_temp)

    return df_temp



def preprocess_all_files(path_to_measurements, parse_time=True, parse_filename=True, concat_files=True, rename_column=True):
  # 
  list_files = glob.glob(path_to_measurements)
  # 
  df_list = []
  for i, f in enumerate(list_files):

    df_temp = preprocess_single_file(f, parse_time=parse_time, parse_filename=parse_filename)
    df_temp['exp_unique_id'] = i

    # print(df_list)
    df_list.append(df_temp)

  
  if concat_files:
      df = pd.concat(df_list)
      col_list = df.columns.tolist()
      new_col_list = [col_list[-1]] + [col_list[-2]]  + col_list[:-2] 
      # print(new_col_list)
      df = df[new_col_list]
      return df

  else: 
      return df_list








def plot_algorithm_performance(names, results, metrics, savefig=False):
    # boxplot algorithm comparison
    fig = plt.figure(figsize=(16,6))
    fig.suptitle('Comparison of algorithm performance')
    ax = fig.add_subplot(111)
    plt.boxplot(results)
    ax.set_xticklabels(names)
    plt.ylabel(metrics)
    plt.xlabel('models')

    if savefig:
        fig.savefig(f'{metrics}.jpeg', transparent = False)
        
    plt.show()




  


import pandas as pd
import subprocess



def load_validated_breath_dataset():


    # s3_url = 's3://raptor-engine/data.csv'
    # filename = './validated_breath_data.csv'
    url = 'https://drive.google.com/file/d/1LUBsw5nIW_VSDGucxeudCrzDy4ntC1gW/view?usp=sharing'
    url='https://drive.google.com/uc?id=' + url.split('/')[-2]


    return pd.read_csv(url, index_col=0)



import pandas as pd
from sklearn.metrics import plot_confusion_matrix
from sklearn.metrics import accuracy_score, precision_score, recall_score, roc_curve, auc, \
        plot_precision_recall_curve
import matplotlib.pyplot as plt
import pandas as pd
import shap
import random

cmap = plt.cm.Blues

def confusion_matrix_plot(model,X_test,X_train, y_train, y_test):
  # 
  y_pred = model.predict(X_test)
  print("Accuracy: %.2f%%" % (accuracy_score(y_test, y_pred) * 100))
  print("Recall: %.2f%%" % (recall_score(y_test, y_pred, average="binary", pos_label='Covid') * 100.0))
  print("Precision: %.2f%%" % (precision_score(y_test, y_pred, average="binary", pos_label='Covid') * 100.0))

  plot_confusion_matrix(model, X_test, y_test,
                            display_labels=['Covid', 'Non-Covid'], cmap=cmap)
  plt.savefig('confusion_matrix.eps')

  fea_imp = pd.DataFrame({'imp': model.feature_importances_, 'col': X_train.columns})
  fea_imp = fea_imp.sort_values(['imp', 'col'], ascending=[True, False]).iloc[-30:]
  fea_imp.plot(kind='barh', x='col', y='imp', figsize=(10, 7), legend=None)
  plt.title('XGBoost - Feature Importance')
  plt.ylabel('Features')
  plt.xlabel('Importance')
  plt.savefig('feature_importance.eps')

  disp = plot_precision_recall_curve(model, X_test, y_test)
  disp.ax_.set_title('Binary class Precision-Recall curve')
  plt.savefig('precision_recall_curve.eps')

def model_explanation(model,X,X_test):
  # 
  explainer = shap.TreeExplainer(model)
  shap_values = explainer.shap_values(X_test) 
  # shap.initjs()
  # 
  # Summary plot
  shap.summary_plot(shap_values, X_test)
  # 
  for name in X.columns:
    shap.dependence_plot(name, shap_values, X_test, display_features = X)


def random_prediction(X_test,y_test,model):
  explainer = shap.TreeExplainer(model)
  shap_values = explainer(X_test)
  i = random.randint(0, len(X_test))
  shap.plots.waterfall(shap_values[i])
  print('Category: ' + str(y_test.values[i][0]))
  print('Prediction: ' + str(model.predict(X_test.iloc[[i]])[0]))
  print('Prediction Probability: ' + str(model.predict_proba(X_test.iloc[[i]])[0]))










import pandas as pd

def read_validated_data_flu_covid(path_to_validated_data):
  # 
  # 12 hours, 24 hours and 48 hours
  # # -------------------------------------------
  # Read data
  # 12 hours
  # 
  # SET 1
  # COVID:
  # 12 hours COVID Control experiments
  df_12h_covid_control_set1_1 = pd.read_csv(path_to_validated_data + 'Flu and COVID RAW Data 03122021/CoV Control 1 12H_13_40_33.47.txt',sep='\t',skiprows=(0,1,2),header=(34))
  df_12h_covid_control_set1_2 = pd.read_csv(path_to_validated_data + 'Flu and COVID RAW Data 03122021/CoV Control 1 12H_13_41_18.47.txt',sep='\t',skiprows=(0,1,2),header=(34))
  df_12h_covid_control_set1_3 = pd.read_csv(path_to_validated_data + 'Flu and COVID RAW Data 03122021/CoV Control 1 12H_13_42_05.47.txt',sep='\t',skiprows=(0,1,2),header=(34))
  df_12h_covid_control_set1_4 = pd.read_csv(path_to_validated_data + 'Flu and COVID RAW Data 03122021/CoV Control 1 12H_13_42_50.47.txt',sep='\t',skiprows=(0,1,2),header=(34))
  # 12 hours COVID experiments
  df_12h_covid_set1_1 = pd.read_csv(path_to_validated_data + 'Flu and COVID RAW Data 03122021/CoV 1 12H_15_16_05.47.txt',sep='\t',skiprows=(0,1,2),header=(34))
  df_12h_covid_set1_2 = pd.read_csv(path_to_validated_data + 'Flu and COVID RAW Data 03122021/CoV 1 12H_15_16_52.47.txt',sep='\t',skiprows=(0,1,2),header=(34))
  df_12h_covid_set1_3 = pd.read_csv(path_to_validated_data + 'Flu and COVID RAW Data 03122021/CoV 1 12H_15_17_39.47.txt',sep='\t',skiprows=(0,1,2),header=(34))
  df_12h_covid_set1_4 = pd.read_csv(path_to_validated_data + 'Flu and COVID RAW Data 03122021/CoV 1 12H_15_18_26.47.txt',sep='\t',skiprows=(0,1,2),header=(34))
  #
  # FLU:
  # 12 hours FLU Control experiments
  df_12h_flu_control_set1_1 = pd.read_csv(path_to_validated_data + 'Flu and COVID RAW Data 03122021/Flu Control 1 12H_14_21_38.47.txt',sep='\t',skiprows=(0,1,2),header=(34))
  df_12h_flu_control_set1_2 = pd.read_csv(path_to_validated_data + 'Flu and COVID RAW Data 03122021/Flu Control 1 12H_14_22_25.47.txt',sep='\t',skiprows=(0,1,2),header=(34))
  # 12 hours FLU experiments
  df_12h_flu_set1_1 = pd.read_csv(path_to_validated_data + 'Flu and COVID RAW Data 03122021/Flu 1 12H_14_43_31.47.txt',sep='\t',skiprows=(0,1,2),header=(34))
  df_12h_flu_set1_2 = pd.read_csv(path_to_validated_data + 'Flu and COVID RAW Data 03122021/Flu 1 12H_14_44_20.47.txt',sep='\t',skiprows=(0,1,2),header=(34))
  df_12h_flu_set1_3 = pd.read_csv(path_to_validated_data + 'Flu and COVID RAW Data 03122021/Flu 1 12H_14_45_07.47.txt',sep='\t',skiprows=(0,1,2),header=(34))
  df_12h_flu_set1_4 = pd.read_csv(path_to_validated_data + 'Flu and COVID RAW Data 03122021/Flu 1 12H_14_45_54.47.txt',sep='\t',skiprows=(0,1,2),header=(34))
  #
  # SET 2
  # COVID:
  # 12 hours COVID Control experiments
  df_12h_covid_control_set2_1 = pd.read_csv(path_to_validated_data + 'Flu and COVID RAW Data 03122021/CoV Control 2 12H_13_44_29.47.txt',sep='\t',skiprows=(0,1,2),header=(34))
  df_12h_covid_control_set2_2 = pd.read_csv(path_to_validated_data + 'Flu and COVID RAW Data 03122021/CoV Control 2 12H_13_45_16.47.txt',sep='\t',skiprows=(0,1,2),header=(34))
  df_12h_covid_control_set2_3 = pd.read_csv(path_to_validated_data + 'Flu and COVID RAW Data 03122021/CoV Control 2 12H_13_46_03.47.txt',sep='\t',skiprows=(0,1,2),header=(34))
  df_12h_covid_control_set2_4 = pd.read_csv(path_to_validated_data + 'Flu and COVID RAW Data 03122021/CoV Control 2 12H_13_46_50.47.txt',sep='\t',skiprows=(0,1,2),header=(34))
  # 12 hours COVID experiments
  df_12h_covid_set2_1 = pd.read_csv(path_to_validated_data + 'Flu and COVID RAW Data 03122021/CoV 2 24H_15_21_02.47.txt',sep='\t',skiprows=(0,1,2),header=(34))
  df_12h_covid_set2_2 = pd.read_csv(path_to_validated_data + 'Flu and COVID RAW Data 03122021/CoV 2 24H_15_21_49.47.txt',sep='\t',skiprows=(0,1,2),header=(34))
  df_12h_covid_set2_3 = pd.read_csv(path_to_validated_data + 'Flu and COVID RAW Data 03122021/CoV 2 24H_15_22_36.47.txt',sep='\t',skiprows=(0,1,2),header=(34))
  #
  # FLU:
  # 12 hours FLU Control experiments
  # [no data] 
  # 12 hours FLU experiments
  df_12h_flu_set2_1 = pd.read_csv(path_to_validated_data + 'Flu and COVID RAW Data 03122021/Flu 2 12H_14_47_37.47.txt',sep='\t',skiprows=(0,1,2),header=(34))
  df_12h_flu_set2_2 = pd.read_csv(path_to_validated_data + 'Flu and COVID RAW Data 03122021/Flu 2 12H_14_48_24.47.txt',sep='\t',skiprows=(0,1,2),header=(34))
  df_12h_flu_set2_3 = pd.read_csv(path_to_validated_data + 'Flu and COVID RAW Data 03122021/Flu 2 12H_14_49_12.47.txt',sep='\t',skiprows=(0,1,2),header=(34))
  #
  # -------------------------------------------
  # Read data
  # 24 hours
  #
  # SET 1
  # COVID:
  # 24 hours COVID Control experiments
  df_24h_covid_control_set1_1 = pd.read_csv(path_to_validated_data + 'Flu and COVID RAW Data 03122021/CoV Control 1 24H_13_48_45.47.txt',sep='\t',skiprows=(0,1,2),header=(34))
  df_24h_covid_control_set1_2 = pd.read_csv(path_to_validated_data + 'Flu and COVID RAW Data 03122021/CoV Control 1 24H_13_50_19.47.txt',sep='\t',skiprows=(0,1,2),header=(34))
  df_24h_covid_control_set1_3 = pd.read_csv(path_to_validated_data + 'Flu and COVID RAW Data 03122021/CoV Control 1 24H_13_51_06.47.txt',sep='\t',skiprows=(0,1,2),header=(34))
  # 24 hours COVID experiments
  df_24h_covid_set1_1 = pd.read_csv(path_to_validated_data + 'Flu and COVID RAW Data 03122021/CoV 1 24H_15_25_04.47.txt',sep='\t',skiprows=(0,1,2),header=(34))
  df_24h_covid_set1_2 = pd.read_csv(path_to_validated_data + 'Flu and COVID RAW Data 03122021/CoV 1 24H_15_25_51.47.txt',sep='\t',skiprows=(0,1,2),header=(34))
  df_24h_covid_set1_3 = pd.read_csv(path_to_validated_data + 'Flu and COVID RAW Data 03122021/CoV 1 24H_15_26_38.47.txt',sep='\t',skiprows=(0,1,2),header=(34))
  #
  # FLU:
  # 24 hours FLU Control experiments
  df_24h_flu_control_set1_1 = pd.read_csv(path_to_validated_data + 'Flu and COVID RAW Data 03122021/Flu Control 1 24H_14_32_09.47.txt',sep='\t',skiprows=(0,1,2),header=(34))
  df_24h_flu_control_set1_2 = pd.read_csv(path_to_validated_data + 'Flu and COVID RAW Data 03122021/Flu Control 1 24H_14_32_56.47.txt',sep='\t',skiprows=(0,1,2),header=(34))
  df_24h_flu_control_set1_3 = pd.read_csv(path_to_validated_data + 'Flu and COVID RAW Data 03122021/Flu Control 1 24H_14_33_43.47.txt',sep='\t',skiprows=(0,1,2),header=(34))
  # 24 hours FLU experiments
  df_24h_flu_set1_1 = pd.read_csv(path_to_validated_data + 'Flu and COVID RAW Data 03122021/Flu 1 24H_14_52_46.47.txt',sep='\t',skiprows=(0,1,2),header=(34))
  df_24h_flu_set1_2 = pd.read_csv(path_to_validated_data + 'Flu and COVID RAW Data 03122021/Flu 1 24H_14_53_33.47.txt',sep='\t',skiprows=(0,1,2),header=(34))
  df_24h_flu_set1_3 = pd.read_csv(path_to_validated_data + 'Flu and COVID RAW Data 03122021/Flu 1 24H_14_54_20.47.txt',sep='\t',skiprows=(0,1,2),header=(34))
  df_24h_flu_set1_4 = pd.read_csv(path_to_validated_data + 'Flu and COVID RAW Data 03122021/Flu 1 24H_14_55_07.47.txt',sep='\t',skiprows=(0,1,2),header=(34))
  #
  # SET 2
  # COVID:
  # 24 hours COVID Control experiments
  df_24h_covid_control_set2_1 = pd.read_csv(path_to_validated_data + 'Flu and COVID RAW Data 03122021/CoV Control 2 24H_13_52_50.47.txt',sep='\t',skiprows=(0,1,2),header=(34))
  df_24h_covid_control_set2_2 = pd.read_csv(path_to_validated_data + 'Flu and COVID RAW Data 03122021/CoV Control 2 24H_13_53_37.47.txt',sep='\t',skiprows=(0,1,2),header=(34))
  df_24h_covid_control_set2_3 = pd.read_csv(path_to_validated_data + 'Flu and COVID RAW Data 03122021/CoV Control 2 24H_13_54_24.47.txt',sep='\t',skiprows=(0,1,2),header=(34))
  df_24h_covid_control_set2_4 = pd.read_csv(path_to_validated_data + 'Flu and COVID RAW Data 03122021/CoV Control 2 24H_13_55_11.47.txt',sep='\t',skiprows=(0,1,2),header=(34))
  # 24 hours COVID experiments
  df_24h_covid_set2_1 = pd.read_csv(path_to_validated_data + 'Flu and COVID RAW Data 03122021/CoV 2 24H_15_21_02.47.txt',sep='\t',skiprows=(0,1,2),header=(34))
  df_24h_covid_set2_2 = pd.read_csv(path_to_validated_data + 'Flu and COVID RAW Data 03122021/CoV 2 24H_15_21_49.47.txt',sep='\t',skiprows=(0,1,2),header=(34))
  df_24h_covid_set2_3 = pd.read_csv(path_to_validated_data + 'Flu and COVID RAW Data 03122021/CoV 2 24H_15_22_36.47.txt',sep='\t',skiprows=(0,1,2),header=(34))
  #
  # FLU:
  # 24 hours FLU Control experiments
  df_24h_flu_control_set2_1 = pd.read_csv(path_to_validated_data + 'Flu and COVID RAW Data 03122021/Flu Control 2 24H_14_28_08.47.txt',sep='\t',skiprows=(0,1,2),header=(34))
  df_24h_flu_control_set2_2 = pd.read_csv(path_to_validated_data + 'Flu and COVID RAW Data 03122021/Flu Control 2 24H_14_28_55.47.txt',sep='\t',skiprows=(0,1,2),header=(34))
  df_24h_flu_control_set2_3 = pd.read_csv(path_to_validated_data + 'Flu and COVID RAW Data 03122021/Flu Control 2 24H_14_29_42.47.txt',sep='\t',skiprows=(0,1,2),header=(34))
  df_24h_flu_control_set2_4 = pd.read_csv(path_to_validated_data + 'Flu and COVID RAW Data 03122021/Flu Control 2 24H_14_30_29.47.txt',sep='\t',skiprows=(0,1,2),header=(34))
  # 24 hours FLU experiments
  df_24h_flu_set2_1 = pd.read_csv(path_to_validated_data + 'Flu and COVID RAW Data 03122021/Flu 2 24H_14_56_50.47.txt',sep='\t',skiprows=(0,1,2),header=(34))
  df_24h_flu_set2_2 = pd.read_csv(path_to_validated_data + 'Flu and COVID RAW Data 03122021/Flu 2 24H_14_57_37.47.txt',sep='\t',skiprows=(0,1,2),header=(34))
  df_24h_flu_set2_3 = pd.read_csv(path_to_validated_data + 'Flu and COVID RAW Data 03122021/Flu 2 24H_14_58_24.47.txt',sep='\t',skiprows=(0,1,2),header=(34))
  df_24h_flu_set2_4 = pd.read_csv(path_to_validated_data + 'Flu and COVID RAW Data 03122021/Flu 2 24H_14_59_12.47.txt',sep='\t',skiprows=(0,1,2),header=(34))

  #
  # -------------------------------------------
  # Read data
  # 48 hours
  #
  # SET 1
  # COVID:
  # 48 hours COVID Control experiments
  df_48h_covid_control_set1_1 = pd.read_csv(path_to_validated_data + 'Flu and COVID RAW Data 03122021/CoV Control 1 48H_13_56_56.47.txt',sep='\t',skiprows=(0,1,2),header=(34))
  df_48h_covid_control_set1_2 = pd.read_csv(path_to_validated_data + 'Flu and COVID RAW Data 03122021/CoV Control 1 48H_13_59_50.47.txt',sep='\t',skiprows=(0,1,2),header=(34))
  df_48h_covid_control_set1_3 = pd.read_csv(path_to_validated_data + 'Flu and COVID RAW Data 03122021/CoV Control 1 48H_14_00_38.47.txt',sep='\t',skiprows=(0,1,2),header=(34))
  df_48h_covid_control_set1_4 = pd.read_csv(path_to_validated_data + 'Flu and COVID RAW Data 03122021/CoV Control 1 48H_14_01_27.47.txt',sep='\t',skiprows=(0,1,2),header=(34))
  # 48 hours COVID experiments
  df_48h_covid_set1_1 = pd.read_csv(path_to_validated_data + 'Flu and COVID RAW Data 03122021/CoV 1 48H_15_27_36.47.txt',sep='\t',skiprows=(0,1,2),header=(34))
  df_48h_covid_set1_2 = pd.read_csv(path_to_validated_data + 'Flu and COVID RAW Data 03122021/CoV 1 48H_15_28_21.47.txt',sep='\t',skiprows=(0,1,2),header=(34))
  df_48h_covid_set1_3 = pd.read_csv(path_to_validated_data + 'Flu and COVID RAW Data 03122021/CoV 1 48H_15_29_08.47.txt',sep='\t',skiprows=(0,1,2),header=(34))
  df_48h_covid_set1_4 = pd.read_csv(path_to_validated_data + 'Flu and COVID RAW Data 03122021/CoV 1 48H_15_29_55.47.txt',sep='\t',skiprows=(0,1,2),header=(34))
  #
  # FLU:
  # 48 hours FLU Control experiments
  df_48h_flu_control_set1_1 = pd.read_csv(path_to_validated_data + 'Flu and COVID RAW Data 03122021/Flu Control 1 48H_14_36_23.47.txt',sep='\t',skiprows=(0,1,2),header=(34))
  df_48h_flu_control_set1_2 = pd.read_csv(path_to_validated_data + 'Flu and COVID RAW Data 03122021/Flu Control 1 48H_14_37_10.47.txt',sep='\t',skiprows=(0,1,2),header=(34))
  df_48h_flu_control_set1_3 = pd.read_csv(path_to_validated_data + 'Flu and COVID RAW Data 03122021/Flu Control 1 48H_14_37_57.47.txt',sep='\t',skiprows=(0,1,2),header=(34))
  # 48 hours FLU experiments
  df_48h_flu_set1_1 = pd.read_csv(path_to_validated_data + 'Flu and COVID RAW Data 03122021/Flu 1 48H_15_00_59.47.txt',sep='\t',skiprows=(0,1,2),header=(34))
  df_48h_flu_set1_2 = pd.read_csv(path_to_validated_data + 'Flu and COVID RAW Data 03122021/Flu 1 48H_15_01_46.47.txt',sep='\t',skiprows=(0,1,2),header=(34))
  #
  # SET 2
  # COVID:
  # 48 hours COVID Control experiments
  df_48h_covid_control_set2_1 = pd.read_csv(path_to_validated_data + 'Flu and COVID RAW Data 03122021/CoV Control 2 48H_14_07_34.47.txt',sep='\t',skiprows=(0,1,2),header=(34))
  df_48h_covid_control_set2_2 = pd.read_csv(path_to_validated_data + 'Flu and COVID RAW Data 03122021/CoV Control 2 48H_14_08_19.47.txt',sep='\t',skiprows=(0,1,2),header=(34))
  # 48 hours COVID experiments
  df_48h_covid_set2_1 = pd.read_csv(path_to_validated_data + 'Flu and COVID RAW Data 03122021/CoV 2 48H_15_35_10.47.txt',sep='\t',skiprows=(0,1,2),header=(34))
  df_48h_covid_set2_2 = pd.read_csv(path_to_validated_data + 'Flu and COVID RAW Data 03122021/CoV 2 48H_15_35_57.47.txt',sep='\t',skiprows=(0,1,2),header=(34))
  df_48h_covid_set2_3 = pd.read_csv(path_to_validated_data + 'Flu and COVID RAW Data 03122021/CoV 2 48H_15_36_46.47.txt',sep='\t',skiprows=(0,1,2),header=(34))
  df_48h_covid_set2_4 = pd.read_csv(path_to_validated_data + 'Flu and COVID RAW Data 03122021/CoV 2 48H_15_37_33.47.txt',sep='\t',skiprows=(0,1,2),header=(34))
  #
  # FLU:
  # 48 hours FLU Control experiments
  df_48h_flu_control_set2_1 = pd.read_csv(path_to_validated_data + 'Flu and COVID RAW Data 03122021/Flu Control 2 48H_14_39_43.47.txt',sep='\t',skiprows=(0,1,2),header=(34))
  df_48h_flu_control_set2_2 = pd.read_csv(path_to_validated_data + 'Flu and COVID RAW Data 03122021/Flu Control 2 48H_14_40_31.47.txt',sep='\t',skiprows=(0,1,2),header=(34))
  df_48h_flu_control_set2_3 = pd.read_csv(path_to_validated_data + 'Flu and COVID RAW Data 03122021/Flu Control 2 48H_14_41_18.47.txt',sep='\t',skiprows=(0,1,2),header=(34))
  # 48 hours FLU experiments
  df_48h_flu_set2_1 = pd.read_csv(path_to_validated_data + 'Flu and COVID RAW Data 03122021/Flu 2 48H_15_07_30.47.txt',sep='\t',skiprows=(0,1,2),header=(34))
  df_48h_flu_set2_2 = pd.read_csv(path_to_validated_data + 'Flu and COVID RAW Data 03122021/Flu 2 48H_15_08_19.47.txt',sep='\t',skiprows=(0,1,2),header=(34))
  df_48h_flu_set2_3 = pd.read_csv(path_to_validated_data + 'Flu and COVID RAW Data 03122021/Flu 2 48H_15_09_06.47.txt',sep='\t',skiprows=(0,1,2),header=(34))
  df_48h_flu_set2_4 = pd.read_csv(path_to_validated_data + 'Flu and COVID RAW Data 03122021/Flu 2 48H_15_09_53.47.txt',sep='\t',skiprows=(0,1,2),header=(34))
  # -------------------------------------------
  # -------------------------------------------  
  # 
  # LISTS
  # Get all the experiments dataframes in a list
  list_all_exp = [
                  df_12h_covid_control_set1_1, df_12h_covid_control_set1_2, df_12h_covid_control_set1_3, df_12h_covid_control_set1_4,
                  df_12h_covid_set1_1, df_12h_covid_set1_2, df_12h_covid_set1_3,
                  df_12h_flu_control_set1_1, df_12h_flu_control_set1_2,
                  df_12h_flu_set1_1, df_12h_flu_set1_2, df_12h_flu_set1_3, df_12h_flu_set1_4,
                  #
                  df_12h_covid_control_set2_1, df_12h_covid_control_set2_2, df_12h_covid_control_set2_3, df_12h_covid_control_set2_4,
                  df_12h_covid_set2_1, df_12h_covid_set2_2, df_12h_covid_set2_3,
                  df_12h_flu_set2_1, df_12h_flu_set2_2, df_12h_flu_set2_3,
                  ##
                  df_24h_covid_control_set1_1, df_24h_covid_control_set1_2, df_24h_covid_control_set1_3,
                  df_24h_covid_set1_1, df_24h_covid_set1_2, df_24h_covid_set1_3,
                  df_24h_flu_control_set1_1, df_24h_flu_control_set1_2, df_24h_flu_control_set1_3,
                  df_24h_flu_set1_1, df_24h_flu_set1_2, df_24h_flu_set1_3, df_24h_flu_set1_4,
                  #
                  df_24h_covid_control_set2_1, df_24h_covid_control_set2_2, df_24h_covid_control_set2_3, df_24h_covid_control_set2_4,
                  df_24h_covid_set2_1, df_24h_covid_set2_2, df_24h_covid_set2_3,
                  df_24h_flu_control_set2_1, df_24h_flu_control_set2_2, df_24h_flu_control_set2_3, df_24h_flu_control_set2_4,
                  df_24h_flu_set2_1, df_24h_flu_set2_2, df_24h_flu_set2_3, df_24h_flu_set2_4,
                  ##
                  df_48h_covid_control_set1_1, df_48h_covid_control_set1_2, df_48h_covid_control_set1_3, df_48h_covid_control_set1_4,
                  df_48h_covid_set1_1, df_48h_covid_set1_2, df_48h_covid_set1_3, df_48h_covid_set1_4,
                  df_48h_flu_control_set1_1, df_48h_flu_control_set1_2, df_48h_flu_control_set1_3,
                  df_48h_flu_set1_1, df_48h_flu_set1_2,
                  #
                  df_48h_covid_control_set2_1, df_48h_covid_control_set2_2, 
                  df_48h_covid_set2_1, df_48h_covid_set2_2, df_48h_covid_set2_3, df_48h_covid_set2_4,
                  df_48h_flu_control_set2_1, df_48h_flu_control_set2_2, df_48h_flu_control_set2_3,
                  df_48h_flu_set2_1, df_48h_flu_set2_2, df_48h_flu_set2_3, df_48h_flu_set2_4
                  ]
  
  # 
  list_all_exp_names = [
                  'df_12h_covid_control_set1_1', 'df_12h_covid_control_set1_2', 'df_12h_covid_control_set1_3', 'df_12h_covid_control_set1_4',
                  'df_12h_covid_set1_1', 'df_12h_covid_set1_2', 'df_12h_covid_set1_3',
                  'df_12h_flu_control_set1_1', 'df_12h_flu_control_set1_2',
                  'df_12h_flu_set1_1', 'df_12h_flu_set1_2', 'df_12h_flu_set1_3', 'df_12h_flu_set1_4',
                  #
                  'df_12h_covid_control_set2_1', 'df_12h_covid_control_set2_2', 'df_12h_covid_control_set2_3', 'df_12h_covid_control_set2_4',
                  'df_12h_covid_set2_1', 'df_12h_covid_set2_2', 'df_12h_covid_set2_3',
                  'df_12h_flu_set2_1', 'df_12h_flu_set2_2', 'df_12h_flu_set2_3',
                  ##
                  'df_24h_covid_control_set1_1', 'df_24h_covid_control_set1_2', 'df_24h_covid_control_set1_3',
                  'df_24h_covid_set1_1', 'df_24h_covid_set1_2', 'df_24h_covid_set1_3',
                  'df_24h_flu_control_set1_1', 'df_24h_flu_control_set1_2', 'df_24h_flu_control_set1_3',
                  'df_24h_flu_set1_1', 'df_24h_flu_set1_2', 'df_24h_flu_set1_3', 'df_24h_flu_set1_4',
                  #
                  'df_24h_covid_control_set2_1', 'df_24h_covid_control_set2_2', 'df_24h_covid_control_set2_3', 'df_24h_covid_control_set2_4',
                  'df_24h_covid_set2_1', 'df_24h_covid_set2_2', 'df_24h_covid_set2_3',
                  'df_24h_flu_control_set2_1', 'df_24h_flu_control_set2_2', 'df_24h_flu_control_set2_3', 'df_24h_flu_control_set2_4',
                  'df_24h_flu_set2_1', 'df_24h_flu_set2_2', 'df_24h_flu_set2_3', 'df_24h_flu_set2_4',
                  ##
                  'df_48h_covid_control_set1_1', 'df_48h_covid_control_set1_2', 'df_48h_covid_control_set1_3', 'df_48h_covid_control_set1_4',
                  'df_48h_covid_set1_1', 'df_48h_covid_set1_2', 'df_48h_covid_set1_3', 'df_48h_covid_set1_4',
                  'df_48h_flu_control_set1_1', 'df_48h_flu_control_set1_2', 'df_48h_flu_control_set1_3',
                  'df_48h_flu_set1_1', 'df_48h_flu_set1_2',
                  #
                  'df_48h_covid_control_set2_1', 'df_48h_covid_control_set2_2', 
                  'df_48h_covid_set2_1', 'df_48h_covid_set2_2', 'df_48h_covid_set2_3', 'df_48h_covid_set2_4',
                  'df_48h_flu_control_set2_1', 'df_48h_flu_control_set2_2', 'df_48h_flu_control_set2_3',
                  'df_48h_flu_set2_1', 'df_48h_flu_set2_2', 'df_48h_flu_set2_3', 'df_48h_flu_set2_4'
                  ]
  # 
  list_12h_exp = [df_12h_covid_control_set1_1, df_12h_covid_control_set1_2, df_12h_covid_control_set1_3, df_12h_covid_control_set1_4,
                  df_12h_covid_set1_1, df_12h_covid_set1_2, df_12h_covid_set1_3,
                  df_12h_flu_control_set1_1, df_12h_flu_control_set1_2,
                  df_12h_flu_set1_1, df_12h_flu_set1_2, df_12h_flu_set1_3, df_12h_flu_set1_4,
                  #
                  df_12h_covid_control_set2_1, df_12h_covid_control_set2_2, df_12h_covid_control_set2_3, df_12h_covid_control_set2_4,
                  df_12h_covid_set2_1, df_12h_covid_set2_2, df_12h_covid_set2_3,
                  df_12h_flu_set2_1, df_12h_flu_set2_2, df_12h_flu_set2_3]
  # 
  list_12h_exp_names = ['df_12h_covid_control_set1_1', 'df_12h_covid_control_set1_2', 'df_12h_covid_control_set1_3', 'df_12h_covid_control_set1_4',
                  'df_12h_covid_set1_1', 'df_12h_covid_set1_2', 'df_12h_covid_set1_3',
                  'df_12h_flu_control_set1_1', 'df_12h_flu_control_set1_2',
                  'df_12h_flu_set1_1', 'df_12h_flu_set1_2', 'df_12h_flu_set1_3', 'df_12h_flu_set1_4',
                  #
                  'df_12h_covid_control_set2_1', 'df_12h_covid_control_set2_2', 'df_12h_covid_control_set2_3', 'df_12h_covid_control_set2_4',
                  'df_12h_covid_set2_1', 'df_12h_covid_set2_2', 'df_12h_covid_set2_3',
                  'df_12h_flu_set2_1', 'df_12h_flu_set2_2', 'df_12h_flu_set2_3']
  # 
  list_24h_exp = [df_24h_covid_control_set1_1, df_24h_covid_control_set1_2, df_24h_covid_control_set1_3,
                  df_24h_covid_set1_1, df_24h_covid_set1_2, df_24h_covid_set1_3,
                  df_24h_flu_control_set1_1, df_24h_flu_control_set1_2, df_24h_flu_control_set1_3,
                  df_24h_flu_set1_1, df_24h_flu_set1_2, df_24h_flu_set1_3, df_24h_flu_set1_4,
                  #
                  df_24h_covid_control_set2_1, df_24h_covid_control_set2_2, df_24h_covid_control_set2_3, df_24h_covid_control_set2_4,
                  df_24h_covid_set2_1, df_24h_covid_set2_2, df_24h_covid_set2_3,
                  df_24h_flu_control_set2_1, df_24h_flu_control_set2_2, df_24h_flu_control_set2_3, df_24h_flu_control_set2_4,
                  df_24h_flu_set2_1, df_24h_flu_set2_2, df_24h_flu_set2_3, df_24h_flu_set2_4]
  # 
  list_24h_exp_names = ['df_24h_covid_control_set1_1', 'df_24h_covid_control_set1_2', 'df_24h_covid_control_set1_3',
                  'df_24h_covid_set1_1', 'df_24h_covid_set1_2', 'df_24h_covid_set1_3',
                  'df_24h_flu_control_set1_1', 'df_24h_flu_control_set1_2', 'df_24h_flu_control_set1_3',
                  'df_24h_flu_set1_1', 'df_24h_flu_set1_2', 'df_24h_flu_set1_3', 'df_24h_flu_set1_4',
                  #
                  'df_24h_covid_control_set2_1', 'df_24h_covid_control_set2_2', 'df_24h_covid_control_set2_3', 'df_24h_covid_control_set2_4',
                  'df_24h_covid_set2_1', 'df_24h_covid_set2_2', 'df_24h_covid_set2_3',
                  'df_24h_flu_control_set2_1', 'df_24h_flu_control_set2_2', 'df_24h_flu_control_set2_3', 'df_24h_flu_control_set2_4',
                  'df_24h_flu_set2_1', 'df_24h_flu_set2_2', 'df_24h_flu_set2_3', 'df_24h_flu_set2_4']
  # 
  list_48h_exp = [ df_48h_covid_control_set1_1, df_48h_covid_control_set1_2, df_48h_covid_control_set1_3, df_48h_covid_control_set1_4,
                  df_48h_covid_set1_1, df_48h_covid_set1_2, df_48h_covid_set1_3, df_48h_covid_set1_4,
                  df_48h_flu_control_set1_1, df_48h_flu_control_set1_2, df_48h_flu_control_set1_3,
                  df_48h_flu_set1_1, df_48h_flu_set1_2,
                  #
                  df_48h_covid_control_set2_1, df_48h_covid_control_set2_2, 
                  df_48h_covid_set2_1, df_48h_covid_set2_2, df_48h_covid_set2_3, df_48h_covid_set2_4,
                  df_48h_flu_control_set2_1, df_48h_flu_control_set2_2, df_48h_flu_control_set2_3,
                  df_48h_flu_set2_1, df_48h_flu_set2_2, df_48h_flu_set2_3, df_48h_flu_set2_4]
  # 
  list_48h_exp_names = ['df_48h_covid_control_set1_1', 'df_48h_covid_control_set1_2', 'df_48h_covid_control_set1_3', 'df_48h_covid_control_set1_4',
                  'df_48h_covid_set1_1', 'df_48h_covid_set1_2', 'df_48h_covid_set1_3', 'df_48h_covid_set1_4',
                  'df_48h_flu_control_set1_1', 'df_48h_flu_control_set1_2', 'df_48h_flu_control_set1_3',
                  'df_48h_flu_set1_1', 'df_48h_flu_set1_2',
                  #
                  'df_48h_covid_control_set2_1', 'df_48h_covid_control_set2_2', 
                  'df_48h_covid_set2_1', 'df_48h_covid_set2_2', 'df_48h_covid_set2_3', 'df_48h_covid_set2_4',
                  'df_48h_flu_control_set2_1', 'df_48h_flu_control_set2_2', 'df_48h_flu_control_set2_3',
                  'df_48h_flu_set2_1', 'df_48h_flu_set2_2', 'df_48h_flu_set2_3', 'df_48h_flu_set2_4']
  # 
  # 
  return list_all_exp, list_12h_exp, list_24h_exp, list_48h_exp, list_all_exp_names, list_12h_exp_names, list_24h_exp_names, list_48h_exp_names


def create_target_validated_data_flu_covid(exp,exp_name):
  for i in range(len(exp)):
    name_str = exp_name[i]
    if ('covid' in name_str) and ('control' in name_str):
      exp[i]['Target'] = 'Control Covid'
    if ('flu' in name_str) and ('control' in name_str):
      exp[i]['Target'] = 'Control Flu'
    elif ('covid' in name_str):
      exp[i]['Target'] = 'Covid'
    elif ('flu' in name_str):
      exp[i]['Target'] = 'Flu'
    else:
      exp[i]['Target'] = 'Control'
  return exp

  






import pandas as pd
import subprocess
import os
from shutil import copyfile
import re
from datetime import datetime

import seaborn as sns
import numpy as np
import itertools
import matplotlib.pyplot as plt
from sklearn.metrics import confusion_matrix
from sklearn.metrics import precision_score, recall_score, accuracy_score

from datetime import datetime
from sklearn.preprocessing import LabelEncoder
from sklearn.manifold import TSNE


from sklearn import linear_model
from sklearn import metrics


# from sklearn.model_selection import GridSearchCV
# from sklearn.svm import LinearSVC
# from sklearn.svm import SVC
# from sklearn.tree import DecisionTreeClassifier
# from sklearn.ensemble import RandomForestClassifier
# from sklearn.ensemble import GradientBoostingClassifier
# from sklearn.model_selection import train_test_split




def get_exp_stage(time, dpps=4, baseline=2, absorb=4, pause=1, desorb=5, flush=18):
    
    # dpps: data point per second

    baseline_time = baseline*dpps
    if time <= baseline_time:
        return 'baseline'
    absorb_time = baseline_time + absorb*dpps
    if time <= absorb_time:
        return 'absorb'
    pause_time = absorb_time + pause*dpps
    if time <= pause_time:
        return 'pause'
    desorb_time = pause_time + desorb*dpps
    if time <= desorb_time:
        return 'desorb'
    flush_time = desorb_time + flush*dpps
    if time <= flush_time:
        return 'flush'
    wait_time = flush_time + flush*dpps
    if time <= wait_time:
        return 'wait'
    
    




def rename_columns(df, has_label=False):
    for col in df.columns:
        if 'Sen' in col:
        # print(col[4:])
            new_col = re.findall('\d+', col)[0]
            # print(new_col)
            df.rename(columns={col:new_col}, inplace=True)
    df.rename(columns={'Data Points': 'data_points'}, inplace=True)
    df.rename(columns={'Humidity (%r.h.)':'humidity'}, inplace=True)
    if has_label:
        df['exp_type'] = df['exp_type'].apply(lambda x: 'Covid' if x == 'COVID' else x)
    return df



def preprocess_data(files, has_label=False, root_dir='./Raw_Data', control_dir='./Control', covid_dir='./Covid'):
    li = []
#     li1 = []
    x = 0
    for f in files:
        if f.endswith(".txt"):
            filename = os.path.join(root_dir, f)
            # exp = !grep "Name of the experiment" "$filename"
            

            exp = subprocess.run(['grep', 'Name of the experiment', filename],
                        check=True, text=True, capture_output=True).stdout

            time = subprocess.run(['grep', '-m', '1', 'Time', filename],
                    check=True, text=True, capture_output=True).stdout

            date = subprocess.run(['grep', '-m', '1', 'Date', filename],
                    check=True, text=True, capture_output=True).stdout

            # time = !grep -m 1 "Time" "$filename"
            # date = !grep -m 1 "Date" "$filename"
        
            
            temp_string = subprocess.run(['awk', 'NR>=39', filename],
                        check=True, text=True, capture_output=True).stdout

            temp_file = open(r'./temp.txt', 'w')
            temp_file.write(temp_string)
            temp_file.close()


            
            
            df = pd.read_csv("temp.txt", sep="\t")
            df['exp_stage'] = df['Data Points'].apply(get_exp_stage)
            
            df['series_id'] = x 
            x += 1

            if has_label:
                exp = subprocess.run(['grep', 'Name of the experiment', filename],
                        check=True, text=True, capture_output=True).stdout

                cult_time, repeat_num, exp_type = exp.split('=')[1].strip().replace('"','').split()
                if exp_type == "COVID":
                    copyfile(filename, os.path.join(covid_dir, f))
                if exp_type == "Control":
                    copyfile(filename, os.path.join(control_dir, f))

                _, _, df['exp_type'] = cult_time, repeat_num, exp_type


            
            df['date'] = date.split('=')[1].strip()
            d = date.split('/n')[0].split('=')[1].strip()
            # print(date)
            # print(d)
            # print(time.split('/n')[0].split('=')[1])

            df['time_start'] = time.split('=')[1].strip()
            df['time_elapsed'] = df.index / 4

            # print(time)
            df['exp_name'] = f

            df['timestamp'] = pd.to_datetime(d + " " + df['time_start'])
           
            df['time'] = pd.to_datetime(d + " " + df['time_start']) + pd.to_timedelta(df['time_elapsed'], unit='s')

            # for i, data in df.groupby('exp_stage'):
                

                # if data['exp_stage'].str.contains('baseline').all():
                #     stage_data_points = data['exp_stage'].str.contains('baseline')
                #     if len(stage_data_points) > 8:
                #         n = len(stage_data_points) - 8
                #         data.drop(data.tail(n).index, inplace=True)
                #         li1.append(data)

                # if data['exp_stage'].str.contains('absorb').all():
                #     stage_data_points = data['exp_stage'].str.contains('absorb')
                #     if len(stage_data_points) > 16:
                #         n = len(stage_data_points) - 16
                #         data.drop(data.tail(n).index, inplace=True)
                #         li1.append(data)

                # if data['exp_stage'].str.contains('pause').all():
                #     stage_data_points = data['exp_stage'].str.contains('pause')
                #     if len(stage_data_points) > 4:
                #         n = len(stage_data_points) - 4
                #         data.drop(data.tail(n).index, inplace=True)
                #         li1.append(data)
                        
                # if data['exp_stage'].str.contains('desorb').all():
                #     stage_data_points = data['exp_stage'].str.contains('desorb')
                #     if len(stage_data_points) > 20:
                #         n = len(stage_data_points) - 20
                #         data.drop(data.tail(n).index, inplace=True)
                #         li1.append(data)

                # if data['exp_stage'].str.contains('flush').all():
                #     stage_data_points = data['exp_stage'].str.contains('flush')
                #     if len(stage_data_points) > 72:
                #         n = len(stage_data_points) - 72
                #         data.drop(data.tail(n).index, inplace=True)
                #         li1.append(data)

                # if data['exp_stage'].str.contains('wait').all():
                #     stage_data_points = data['exp_stage'].str.contains('wait')
                #     if len(stage_data_points) > 20:
                #         n = len(stage_data_points) - 20
                #         data.drop(data.tail(n).index, inplace=True)
                #         li1.append(data)
                
            
            li.append(df)


    frame = pd.concat(li, axis=0, ignore_index=True)
    frame  = rename_columns(frame, has_label)
    frame = frame.set_index('time')

    # frame1 = pd.concat(li1, axis=0, ignore_index=True)
    # frame1  = rename_columns(frame1, has_label)
    # frame1 = frame1.set_index('time')

    os.remove('temp.txt')
    return frame


# def perform_tsne(X_data, y_data, perplexities, markers, n_iter=1000):
    
#     for index, perplexity in enumerate(perplexities):
#         # perform t-sne
#         print("\nPerforming tsne with perplexity {} and with {} iterations at max".format(perplexity, n_iter))
#         X_reduced = TSNE(verbose=2, perplexity=perplexity).fit_transform(X_data)
#         print('Done..')
        
#         # prepare data for seaborn
#         print("Creating plot for this t-sne visualization")
#         df = pd.DataFrame({'x':X_reduced[:,0], 'y':X_reduced[:,1], 'label':y_data})
        
#         # draw the plot in appropriate palce in the grid
#         sns.lmplot(data=df, x='x', y='y', hue='label', fit_reg=False, height=8, palette="Set1",markers=markers)
#         plt.title("perplexity : {} and max_iter: {}".format(perplexity, n_iter))
#         plt.show()
#         print("Done")
#     return X_reduced





# def plot_confusion_matrix(cm, classes,
#                         normalize=False,
#                         title='Confusion matrix',
#                         cmap=plt.cm.Blues):
#     if normalize:
#         cm = cm.astype('float') / cm.sum(axis=1)[:, np.newaxis]

#     plt.imshow(cm, interpolation='nearest', cmap=cmap)
#     plt.title(title)
#     plt.colorbar()
#     tick_marks = np.arange(len(classes))
#     plt.xticks(tick_marks, classes, rotation=90)
#     plt.yticks(tick_marks, classes)

#     fmt = '.2f' if normalize else 'd'
#     thresh = cm.max() / 2.
#     for i, j in itertools.product(range(cm.shape[0]), range(cm.shape[1])):
#         plt.text(j, i, format(cm[i, j], fmt),
#                 horizontalalignment="center",
#                 color="white" if cm[i, j] > thresh else "black")

#     plt.tight_layout()
#     plt.ylabel('True label')
#     plt.xlabel('Predicted label')



def model_run(model, X_train, y_train, X_test, y_test, class_labels, cm_normalize='all', \
                 print_cm=True, cm_cmap=plt.cm.Blues):
    
    
    # to store results at various phases
    results = dict()
    
    # time at which model starts training 
    train_start_time = datetime.now()
    print('training the model..')
    model.fit(X_train, y_train)
    print('Done....!\n')
    train_end_time = datetime.now()
    results['training_time'] =  train_end_time - train_start_time
    print('==> training time:- {}\n'.format(results['training_time']))
    
    
    # predict test data
    print('Predicting test data')
    test_start_time = datetime.now()
    y_pred = model.predict(X_test)
    test_end_time = datetime.now()
    print('Done....!\n')
    results['testing_time'] = test_end_time - test_start_time
    print('==> testing time:- {}\n'.format(results['testing_time']))
    results['predicted'] = y_pred
   

    # calculate overall accuracy, precision and recall of the model
    accuracy = accuracy_score(y_test, y_pred)
    precision = precision_score(y_test, y_pred)
    recall = recall_score(y_test, y_pred)



    # store accuracy, precision and recall in results
    results['accuracy'] = accuracy
    results['precision'] = precision
    results['recall'] = recall

    print('==> Accuracy:- {}\n'.format(accuracy))
    print('==> Recall:- {}\n'.format(recall))
    print('==> Precision:- {}\n'.format(precision))


   
    
    
    # # confusion matrix
    cm = metrics.confusion_matrix(y_test, y_pred, normalize='all')
    # results['confusion_matrix'] = cm
    # if print_cm: 
    #     print('\n ********Confusion Matrix********')
    #     print('\n {}'.format(cm))
        
    # # plot confusin matrix
    # plt.figure(figsize=(6,6))
    # plt.grid(b=False)
    # plot_confusion_matrix(cm, classes=class_labels, normalize='all', title='Normalized confusion matrix', cmap = cm_cmap)
    # plt.show()





    ax= plt.subplot()
    sns.heatmap(cm, annot=True, fmt='.3g', ax=ax);  #annot=True to annotate cells, ftm='g' to disable scientific notation

    # labels, title and ticks
    ax.set_xlabel('Predicted labels');ax.set_ylabel('True labels'); 
    ax.set_title('Confusion Matrix'); 
    ax.xaxis.set_ticklabels(['Covid', 'Control']); ax.yaxis.set_ticklabels(['Covid', 'Control']);
    

    # target_names = ['Covid', 'Control']



    # cmn = cm.astype('float') / cm.sum(axis=1)#[:, np.newaxis]
    # fig, ax = plt.subplots(figsize=(10,10))
    # sns.heatmap(cmn, annot=True, fmt='.2f', xticklabels=target_names, yticklabels=target_names)
    # plt.ylabel('Actual')
    # plt.xlabel('Predicted')
    # plt.show(block=False)



    # get classification report
    print('****************| Classifiction Report |****************')
    classification_report = metrics.classification_report(y_test, y_pred)
   
    # store report in results
    results['classification_report'] = classification_report
    print(classification_report)
    
    # add the trained  model to the results
    results['model'] = model
    
    return results
    






def print_grid_search_attributes(model):
    # Estimator that gave highest score among all the estimators formed in GridSearch
    print('\n\n==> Best Estimator:')
    print('\t{}\n'.format(model.best_estimator_))


    # parameters that gave best results while performing grid search
    print('\n==> Best parameters:')
    print('\tParameters of best estimator : {}'.format(model.best_params_))


    #  number of cross validation splits
    print('\n==> No. of CrossValidation sets:')
    print('\tTotal numbre of cross validation sets: {}'.format(model.n_splits_))


    # Average cross validated score of the best estimator, from the Grid Search 
    print('\n==> Best Score:')
    print('\tAverage Cross Validate scores of best estimator : {}'.format(model.best_score_))



def plot_algorithm_performance(names, results, metrics, savefig=False, ylim=False):
# boxplot algorithm comparison
    fig = plt.figure(figsize=(16,6))
    fig.suptitle('Comparison of algorithm performance')
    ax = fig.add_subplot(111)
    plt.boxplot(results)
    ax.set_xticklabels(names)
    plt.ylabel(metrics)
    plt.xlabel('models')

    if savefig:
        fig.savefig(f'{metrics}.jpeg', transparent = False)
    
    if ylim:
        plt.ylim([0,1])
        
    plt.show()



def run_models_cv(models, X_train, y_train, scoring='accuracy'):
    results = []
    names = []

    if scoring == 'specificity':
        scoring = make_scorer(recall_score, pos_label=0)
    else:
        for name, model in models:
            kfold = model_selection.KFold(n_splits=10, random_state=seed, shuffle=True)
            cv_results = model_selection.cross_val_score(model, X_train, y_train, cv=kfold, scoring=scoring)
            results.append(cv_results)
            names.append(name)



def plot_sensor_signals(df, sensors_signals_col, target_col_name):
        
    facetgrid = sns.FacetGrid(df, hue=target_col_name, height=5,aspect=3)
    facetgrid.map(sns.distplot, sensors_signals_col, hist=False).add_legend()


def signals_plot_by_target(df, target_col_name, labels):
    fig, axes = plt.subplots(nrows=24,ncols=2, figsize=(20,36))
    df[df[target_col_name]==labels[0]].iloc[:, 1:25].plot(ax=axes[:,0], subplots=True, sharex=True, figsize=(10,25))

    df[df[target_col_name]==labels[1]].iloc[:, 1:25].plot(ax=axes[:,1], subplots=True, sharex=True, figsize=(10,25));

    # plt.legend()

    axes[0][0].set_title(labels[0])
    axes[0][1].set_title(labels[1])

    plt.show();




def plot_by_exp_stage():
    fig, axes = plt.subplots(nrows=24,ncols=5, figsize=(20,36))
    df[df['exp_stage']=='baseline'].iloc[:, 1:25].plot(ax=axes[:,0], subplots=True, sharex=True, figsize=(10,25))

    df[df['exp_stage']=='absorb'].iloc[:, 1:25].plot(ax=axes[:,1], subplots=True, sharex=True, figsize=(10,25));
    df[df['exp_stage']=='pause'].iloc[:, 1:25].plot(ax=axes[:,2], subplots=True, sharex=True, figsize=(10,25));
    df[df['exp_stage']=='desorb'].iloc[:, 1:25].plot(ax=axes[:,3], subplots=True, sharex=True, figsize=(10,25));
    df[df['exp_stage']=='flush'].iloc[:, 1:25].plot(ax=axes[:,4], subplots=True, sharex=True, figsize=(10,25));

    # plt.legend()

    axes[0][0].set_title('baseline')
    axes[0][1].set_title('absorb')
    axes[0][2].set_title('pause')
    axes[0][3].set_title('desorb')
    axes[0][4].set_title('flush')





def plot_new_sample(model, x_train, x):
    fig, ax = plt.subplots()
    x = x.reshape(1,-1)
    ax.scatter(x_train[:,0], x_train[:,1], c=l)
    ax.scatter(x[:,0], x[:,1])